@@ -1,7 +1,8 @@
-#!/usr/bin/env python
-# ./collect_logs.py log_foo.txt log_bar.txt ...
-#
-import sys
+"""
+Author: Noritaka Shimizu (?)
+Modified by: Jon Dahl
+"""
+import sys, os, warnings
 from typing import List, Tuple
 from fractions import Fraction
 from math import pi
@@ -9,7 +10,6 @@ from math import pi
 #weisskopf_threshold = 1.0 # threshold to show in W.u.
 weisskopf_threshold = -0.001
 
-e_data = {}     # e_data[energy] = (log filename, spin, parity, eigenstate number, tt).
 n_jnp = {}
 
 E_gs = 0.0
@@ -70,10 +70,10 @@ def weisskopf_unit(multipole_type: str, mass: int) -> Tuple[float, str]:
     
     return B_weisskopf, unit_weisskopf
     
-def read_energy_logfile(filename: str):
+def read_energy_logfile(filename: str, E_data: dict):
     """
     Extract the energy, spin, and parity for each eigenstate and arrange
-    the data in a dictionary, e_data, where the keys are the energies
+    the data in a dictionary, E_data, where the keys are the energies
     and the values are tuples of
     (log filename, spin, parity, eigenstate number, tt).
 
@@ -86,7 +86,6 @@ def read_energy_logfile(filename: str):
     filename : str
         The log filename.
     """
-    # infile = open(filename, 'r')
     with open(filename, "r") as infile:
         while True:
             line = infile.readline()
@@ -114,12 +113,12 @@ def read_energy_logfile(filename: str):
                     spin = int(line[45:48])     # 2*spin actually.
                     parity = int(line[57:59])
                     parity = parity_integer_to_string(parity)
-                    while energy in e_data: energy += 0.000001  # NOTE: To separate energies close together? Else keys may be identical!
+                    while energy in E_data: energy += 0.000001  # NOTE: To separate energies close together? Else keys may be identical!
                     while True:
                         line = infile.readline()
                         if line[42:45] != ' T:': continue
                         tt = int(line[45:48])
-                        e_data[ energy ] = (filename, spin, parity, n_eig, tt)
+                        E_data[ energy ] = (filename, spin, parity, n_eig, tt)
                         break
 
 def spin_to_string(spin: int) -> str:
@@ -151,7 +150,10 @@ def spin_to_string(spin: int) -> str:
 
     return res
 
-def read_transit_logfile_old(filename: str, multipole_type: str):
+def read_transit_logfile_old(
+    filename: str,
+    multipole_type: str
+    ):
     """
     Extract transit information from transit logfile. Old syntax style,
     pre 2021-11-24.
@@ -169,6 +171,10 @@ def read_transit_logfile_old(filename: str, multipole_type: str):
     ------
     Exception:
         If mass != mass_save. Unsure why mass_save is here at all.
+
+    ValueError:
+        If the conversion of log file values to int or float cannot be
+        done. This indicates wrong log syntax.
     """
     out_e = {}
     mass_save = 0           # NOTE: Unclear what this is used for.
@@ -261,29 +267,27 @@ def read_transit_logfile_old(filename: str, multipole_type: str):
                 """
                 continue
             
-            # spin_final   = int(line_split[0])
-            # idx_1        = int(line_split[1])
-            # E_final      = float(line_split[2]) - E_gs
-            # spin_initial = int(line_split[3])
-            # idx_2        = int(line_split[4])
-            # E_initial    = float(line_split[5]) - E_gs
-            # dE           = float(line_split[6])
-            # Mred         = float(line_split[7])
-            # B_decay      = float(line_split[8])
-            # B_excite     = float(line_split[9])
-            # Mom          = float(line_split[10])
-            spin_final = int(line[:2])
-            idx_1 = int(line[3:7])
-            spin_initial = int(line[17:19])
-            idx_2 = int(line[20:24])
-            dE = float(line[34:42]) # Gamma energy.
-            E_final = float(line[8:17]) - E_gs
-            E_initial = float(line[25:34]) - E_gs
-            B_decay = float(line[52:62])
-            B_excite = float(line[62:72])
+            try:
+                spin_final = int(line[:2])
+                idx_1 = int(line[3:7])
+                spin_initial = int(line[17:19])
+                idx_2 = int(line[20:24])
+                dE = float(line[34:42]) # Gamma energy.
+                E_final = float(line[8:17]) - E_gs
+                E_initial = float(line[25:34]) - E_gs
+                B_decay = float(line[52:62])
+                B_excite = float(line[62:72])
+            except ValueError as err:
+                msg = f"\n{err.__str__()}"
+                msg += "\nThis might be due to wrong log file syntax."
+                msg += " Try using old_or_new='new' or 'both' as argument"
+                msg += " to collect_logs."
+                msg += f"\n{filename = }"
+                msg += f"\n{line = }"
+                raise ValueError(msg)
+
             B_weisskopf_decay  = B_decay/B_weisskopf
             B_weisskopf_excite = B_excite/B_weisskopf
-
             if (spin_final == spin_initial) and (idx_1 == idx_2): continue
             if is_diag and (dE < 0.0): continue
             if (B_weisskopf_decay < weisskopf_threshold): continue
@@ -314,11 +318,6 @@ def read_transit_logfile_old(filename: str, multipole_type: str):
                 NOTE: What is this option used for? In what case is the
                 excitation energy negative?
                 """
-                # out = stringformat \
-                #     % (spin_to_string(spin_final), parity_final, idx_1, E_final, 
-                #         spin_to_string(spin_initial), parity_initial, idx_2, E_initial, 
-                #         -dE, B_decay, B_weisskopf_decay, B_excite, B_weisskopf_excite)
-
                 out = f"{spin_to_string(spin_final):4s} "
                 out += f"{parity_final:1s} "
                 out += f"{idx_1:4d} "
@@ -354,9 +353,14 @@ def read_transit_logfile(filename: str, multipole_type: str):
     ------
     Exception:
         If mass != mass_save. Unsure why mass_save is here at all.
+
+    ValueError:
+        If the conversion of log file values to int or float cannot be
+        done. This indicates wrong log syntax.
     """
     out_e = {}
     mass_save = 0           # NOTE: Unclear what this is used for.
+    unit_weisskopf = None   # Stays None if the logfile is invalid.
     with open(filename, "r") as infile:
         for line in infile:
             """
@@ -445,18 +449,27 @@ def read_transit_logfile(filename: str, multipole_type: str):
                 ...
                 """
                 continue
-            
-            spin_final   = int(line_split[0])
-            idx_1        = int(line_split[1])
-            E_final      = float(line_split[2]) - E_gs
-            spin_initial = int(line_split[3])
-            idx_2        = int(line_split[4])
-            E_initial    = float(line_split[5]) - E_gs
-            dE           = float(line_split[6])
-            Mred         = float(line_split[7])
-            B_decay      = float(line_split[8])
-            B_excite     = float(line_split[9])
-            Mom          = float(line_split[10])
+            try:
+                spin_final   = int(line_split[0])
+                idx_1        = int(line_split[1])
+                E_final      = float(line_split[2]) - E_gs
+                spin_initial = int(line_split[3])
+                idx_2        = int(line_split[4])
+                E_initial    = float(line_split[5]) - E_gs
+                dE           = float(line_split[6])
+                Mred         = float(line_split[7])
+                B_decay      = float(line_split[8])
+                B_excite     = float(line_split[9])
+                Mom          = float(line_split[10])
+            except ValueError as err:
+                msg = f"\n{err.__str__()}"
+                msg += "\nThis might be due to wrong log file syntax."
+                msg += " Try using old_or_new='old' or 'both' as argument to"
+                msg += " collect_logs."
+                msg += f"\n{filename = }"
+                msg += f"\n{line = }"
+                raise ValueError(msg)
+
             B_weisskopf_decay  = B_decay/B_weisskopf
             B_weisskopf_excite = B_excite/B_weisskopf
 
@@ -490,10 +503,6 @@ def read_transit_logfile(filename: str, multipole_type: str):
                 NOTE: What is this option used for? In what case is the
                 excitation energy negative?
                 """
-                # out = stringformat \
-                #     % (spin_to_string(spin_final), parity_final, idx_1, E_final, 
-                #         spin_to_string(spin_initial), parity_initial, idx_2, E_initial, 
-                #         -dE, B_decay, B_weisskopf_decay, B_excite, B_weisskopf_excite)
                 out = f"{spin_to_string(spin_final):4s} "
                 out += f"{parity_final:1s} "
                 out += f"{idx_1:4d} "
@@ -512,53 +521,275 @@ def read_transit_logfile(filename: str, multipole_type: str):
 
     return unit_weisskopf, out_e, mass_save
 
-def print_transition(multipole_type):
-    is_show = False
-    output_e = {}
+def check_multipolarities(path: str="."):
+    """
+    Check whether E1, M1, and E2 values are present in transit logfiles.
+    Currently only checks that the correct headers are there, not the
+    actual values.
+
+    Parameters
+    ----------
+    path : str
+        Path to directory with transit logfiles.
+    """
+    print("logfile                                 E1        M1        E2        Is it correct?")
+    print("------------------------------------------------------------------------------------")
+    # print("log_V50_sdpf-mu_tr_j12n_j16n.txt     True   False  True")
+    for elem in os.listdir(path):
+        E1 = False
+        should_it_have_E1 = False
+        M1 = False
+        should_it_have_M1 = False
+        E2 = False
+        should_it_have_E2 = False
+
+        if elem.startswith("log_") and elem.endswith(".txt") and ("_tr_" in elem):
+            tmp = elem[:-4]
+            tmp = tmp.split("_")
+            orbital_1_parity = tmp[-2][-1]
+            orbital_2_parity = tmp[-1][-1]
+            orbital_1_spin = int(tmp[-2][1:-1])/2
+            orbital_2_spin = int(tmp[-1][1:-1])/2
+            # spin_diff = abs(orbital_1_spin - orbital_2_spin)
+            spin_range = list(range(
+                int(abs(orbital_1_spin - orbital_2_spin)),
+                int(orbital_1_spin + orbital_2_spin + 1),
+                1
+            ))
+
+            if orbital_1_parity == orbital_2_parity:
+                if (1 in spin_range) and (2 not in spin_range):
+                    should_it_have_E1 = False
+                    should_it_have_M1 = True
+                    should_it_have_E2 = False
+                elif (1 in spin_range) and (2 in spin_range):
+                    should_it_have_E1 = False
+                    should_it_have_M1 = True
+                    should_it_have_E2 = True
+                elif (1 not in spin_range) and (2 in spin_range):
+                    should_it_have_E1 = False
+                    should_it_have_M1 = False
+                    should_it_have_E2 = True
+                else:
+                    msg = "No conditions met! (parity change)"
+                    raise RuntimeError(msg)
+
+            elif orbital_1_parity != orbital_2_parity:
+                if (1 in spin_range) and (2 not in spin_range):
+                    should_it_have_E1 = True
+                    should_it_have_M1 = False
+                    should_it_have_E2 = False
+                elif (1 in spin_range) and (2 in spin_range):
+                    should_it_have_E1 = True
+                    should_it_have_M1 = False
+                    should_it_have_E2 = False
+                elif (1 not in spin_range) and (2 in spin_range):
+                    """
+                    This condition should never be met since KSHELL
+                    will never calculate these transitions.
+                    """
+                    should_it_have_E1 = False
+                    should_it_have_M1 = False
+                    should_it_have_E2 = False
+                    msg = "This condition should not ever be met!"
+                    raise RuntimeError(msg)
+                else:
+                    msg = "No conditions met! (no parity change)"
+                    raise RuntimeError(msg)
+            else:
+                msg = "No conditions met! (could not decide parity change)"
+                raise RuntimeError(msg)
+
+            with open(f"{path}/{elem}", "r") as infile:
+                for line in infile:
+                    if "E1 transition" in line:
+                        E1 = True
+                    elif "M1 transition" in line:
+                        M1 = True
+                    elif "E2 transition" in line:
+                        E2 = True
+                
+                if (E1 == should_it_have_E1) and (M1 == should_it_have_M1) and (E2 == should_it_have_E2):
+                    correct = True
+                else:
+                    correct = False
+                print(f"{elem:40s}{str(E1):10s}{str(M1):10s}{str(E2):10s}{correct}")
+
+def collect_logs(path: str=".", old_or_new: str="new"):
+    """
+    Collect energy and transition data from all log files in 'path'.
+
+    Parameters
+    ----------
+    path : str
+        Path to directory with log files, or parth to single energy log
+        file, if you wanna do that for some reason.
+
+    old_or_new : str
+        Choose between old or new log file syntax. Pre or post
+        2021-11-24. Summary file syntax is of new type, regardless.
+        The option "both" is for the rare case of log files of mixed
+        syntax in the same directory.
+
+    Raises
+    ------
+    RuntimeError:
+        If no energy log files are found in 'path'.
+
+    ValueError:
+        If 'old_or_new' is of invalid value.
+    """
+    allowed_old_or_new = ["new", "old", "both"]
+    old_or_new = old_or_new.lower()
+    if old_or_new not in allowed_old_or_new:
+        msg = f"old_or_new must be in {allowed_old_or_new}. Got {old_or_new}."
+        raise ValueError(msg)
+
+    energy_log_files = []
+    transit_log_files = []
+    isotopes = []
+    for elem in os.listdir(path):
+        # if elem.startswith("log_") and elem.endswith(".txt"):
+        if ("log_" in elem) and elem.endswith(".txt"):
+            tmp = elem.split("_")
+            isotope_index = tmp.index("log") + 1    # Isotope name is always after 'log'.
+            # if (tmp := elem.split("_")[1].lower()) not in isotopes:
+            if tmp[isotope_index] not in isotopes:
+                isotopes.append(tmp[isotope_index])
+            if not "_tr_" in elem:
+                energy_log_files.append(f"{path}/{elem}")
+            elif "_tr_" in elem:
+                transit_log_files.append(f"{path}/{elem}")
     
-    for filename in sys.argv[1:]:
-        unit_weisskopf, out_e, mass = read_transit_logfile(filename, multipole_type)
-        if unit_weisskopf: is_show = unit_weisskopf
-        output_e.update(out_e)
+    if len(isotopes) > 1:
+        print(f"Log files for different isotopes have been found in {path}")
+        msg = f"Found: {isotopes}. Your choice: "
+        while True:
+            choice = input(msg)
+            if choice in isotopes:
+                break
+        
+        # Remove all log files not of type 'choice'.
+        energy_log_files = [i for i in energy_log_files if choice in i.split("/")[-1].lower()]
+        transit_log_files = [i for i in transit_log_files if choice in i.split("/")[-1].lower()]
+
+    if len(energy_log_files) == 0:
+        msg = f"No energy log files in path '{path}'."
+        raise FileNotFoundError(msg)
+
+    if len(transit_log_files) == 0:
+        msg = f"No transit log files in path '{path}', only energy log files."
+        warnings.warn(msg, RuntimeWarning)
+
+    E_data = {} # E_data[energy] = (log filename, spin, parity, eigenstate number, tt).
+    spin_parity_occurrences = {}    # Count the occurrences of each (spin, parity) pair.
+    multipole_types = ["E1", "M1", "E2"]
     
-    B_weisskopf, unit_weisskopf = weisskopf_unit(multipole_type, mass)
-    output = f"B({multipole_type})  ( > {weisskopf_threshold:.1f} W.u.)  mass = {mass}    1 W.u. = {B_weisskopf:.1f} {unit_weisskopf}"
-    output += f"\n{is_show} (W.u.)"
-    output += f"\nJ_i  pi_i idx_i Ex_i    J_f  pi_f idx_f Ex_f      dE         B({multipole_type})->         B({multipole_type})->[wu]     B({multipole_type})<-         B({multipole_type})<-[wu]\n"
+    print("Loading energy log files...")
+    for i, log_file in enumerate(energy_log_files):
+        progress_message = f"Reading file {i + 1:3d} of {len(energy_log_files)},"
+        progress_message += f" '{log_file.split('/')[-1]}'"
+        print(progress_message)
+        read_energy_logfile(log_file, E_data)
 
-    for _, out in sorted(output_e.items()):
-        output += out        
-    if is_show: print(output)
+    energies = E_data.keys()
+    if len(energies) == 0:
+        msg = "No energy data has been read from energy logs!"
+        raise RuntimeError(msg)
 
-def main(filename_list: List):
-    for filename in filename_list:
-        read_energy_logfile(filename)
+    energies = sorted(energies)
+    for energy in energies:
+        """
+        What does this loop actually do...?
+        """
+        filename, spin, parity, n_eig, tt = E_data[energy]
+        spin_parity = (spin, parity)
+        try:
+            """
+            Count the number of each (spin, parity) occurrence.
+            """
+            spin_parity_occurrences[spin_parity] += 1
+        except KeyError:
+            """
+            Create initial value if key has not yet occurred.
+            """
+            spin_parity_occurrences[spin_parity] = 1
+        n_jnp[ (spin, parity, n_eig) ] = spin_parity_occurrences[spin_parity]
+        E_data[energy] = filename, spin, parity, spin_parity_occurrences[spin_parity], tt
+    
+    global E_gs
+    E_gs = energies[0]
 
-    print("\n Energy levels")
-    keys = e_data.keys()
-    if len(keys) > 0:
-        keys = sorted(keys)
-        njp = {}
-        for e in keys:
-            filename, mtot, prty, n_eig, tt = e_data[e]
-            mp = (mtot, prty)
-            njp[mp] = njp.get(mp, 0) + 1
-            n_jnp[ (mtot, prty, n_eig) ] = njp[mp]
-            e_data[e] = filename, mtot, prty, njp[mp], tt
+    counter = 0
+    filename_without_path = energy_log_files[0].split("/")[-1]
+    isotope = filename_without_path.split("_")[1]
+    model_space = filename_without_path.split("_")[2]
+    while True:
+        """
+        Create unique summary filename.
+        """
+        summary_filename = f"summary_{isotope}_{model_space}_{counter:03d}"
+        if os.path.isfile(f"{path}/{summary_filename}.txt"):
+            counter += 1
+        else:
+            summary_filename += ".txt"
+            break
+    
+    with open(f"{path}/{summary_filename}", "w") as outfile:
+        outfile.write("\n Energy levels\n")
+        outfile.write('\n    N   J     prty N_Jp T        E(MeV)    Ex(MeV)  log-file\n\n')
+        for i, energy in enumerate(energies):
+            filename, spin, parity, n_eig, tt = E_data[energy]
+            out = f"{i + 1:5d}   "
+            out += f"{spin_to_string(spin):5s} "
+            out += f"{parity:1s} "
+            out += f"{n_eig:5d}   "
+            out += f"{spin_to_string(tt):3s} "
+            out += f"{energy:10.3f} "
+            out += f"{energy - E_gs:10.3f}   "
+            out += f"{filename}\n"
+            outfile.write(out)
+        outfile.write("\n")
 
-        global E_gs
-        E_gs = keys[0]
-        print('\n    N    J prty N_Jp    T     E(MeV)  Ex(MeV)  log-file\n')
-        for i, e in enumerate(keys):
-            filename, mtot, prty, n_eig, tt = e_data[e]
-            print("%5d %5s %1s %5d %5s %10.3f %8.3f  %s " \
-                % (i+1, spin_to_string(mtot), prty, n_eig, spin_to_string(tt), e, e-E_gs, filename))
-        print()
+        if len(transit_log_files) > 0:
+            print("\nLoading transit log files...")
+            for multipole_type in multipole_types:
+                output_e = {}
+                
+                for i, filename in enumerate(transit_log_files):
+                    progress_message = f"Reading file {i + 1:3d} of {len(transit_log_files)},"
+                    progress_message += f" multipole: {multipole_type},"
+                    progress_message += f" '{filename.split('/')[-1]}'"
+                    print(progress_message)
+                    if old_or_new == "new":
+                        unit_weisskopf, out_e, mass = read_transit_logfile(filename, multipole_type)
+                    elif old_or_new == "old":
+                        unit_weisskopf, out_e, mass = read_transit_logfile_old(filename, multipole_type)
+                    elif old_or_new == "both":
+                        try:
+                            unit_weisskopf, out_e, mass = read_transit_logfile_old(filename, multipole_type)
+                        except ValueError:
+                            """
+                            If a ValueError is raised inside
+                            read_transit_logfile_old, it is due to the
+                            log file being new syntax styled.
+                            """
+                            unit_weisskopf, out_e, mass = read_transit_logfile(filename, multipole_type)
+                    
+                    if unit_weisskopf is None:
+                        """
+                        Incomplete and invalid logfile.
+                        """
+                        print(f"Incomplete logfile '{filename}'. Skipping...")
+                        continue
+                    
+                    output_e.update(out_e)
+                
+                B_weisskopf, unit_weisskopf = weisskopf_unit(multipole_type, mass)
+                outfile.write(f"B({multipole_type})  ( > {weisskopf_threshold:.1f} W.u.)  mass = {mass}    1 W.u. = {B_weisskopf:.1f} {unit_weisskopf}")
+                outfile.write(f"\n{unit_weisskopf} (W.u.)")
+                outfile.write(f"\nJ_i  pi_i idx_i Ex_i    J_f  pi_f idx_f Ex_f      dE         B({multipole_type})->         B({multipole_type})->[wu]     B({multipole_type})<-         B({multipole_type})<-[wu]\n")
 
-    print_transition('E2')
-    print_transition('M1')
-    print_transition('E1')
-    # print_transition('E3')
-
-if __name__ == "__main__":
-    main(sys.argv[1:])
\ No newline at end of file
+                for _, out in sorted(output_e.items()):
+                    outfile.write(out)
+                outfile.write("\n\n")
\ No newline at end of file
